/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module appstream.Pool;

private import appstream.ComponentBox;
private import appstream.c.functions;
public  import appstream.c.types;
private import gio.AsyncResultIF;
private import gio.Cancellable;
private import glib.ConstructionException;
private import glib.ErrorG;
private import glib.GException;
private import glib.Str;
private import glib.c.functions;
private import gobject.ObjectG;
private import gobject.Signals;
private import std.algorithm;


/** */
public class Pool : ObjectG
{
	/** the main Gtk struct */
	protected AsPool* asPool;

	/** Get the main Gtk struct */
	public AsPool* getPoolStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return asPool;
	}

	/** the main Gtk struct as a void* */
	protected override void* getStruct()
	{
		return cast(void*)asPool;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (AsPool* asPool, bool ownedRef = false)
	{
		this.asPool = asPool;
		super(cast(GObject*)asPool, ownedRef);
	}


	/** */
	public static GType getType()
	{
		return as_pool_get_type();
	}

	/**
	 * Creates a new #AsPool.
	 *
	 * Returns: a #AsPool
	 *
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this()
	{
		auto __p = as_pool_new();

		if(__p is null)
		{
			throw new ConstructionException("null returned by new");
		}

		this(cast(AsPool*) __p, true);
	}

	/**
	 * Register a set of components with the pool temporarily.
	 * Data from components added like this will not be cached.
	 *
	 * Params:
	 *     cbox = Components to add to the pool.
	 *
	 * Returns: %TRUE if the new components were successfully added to the pool.
	 *
	 * Since: 0.15.0
	 *
	 * Throws: GException on failure.
	 */
	public bool addComponents(ComponentBox cbox)
	{
		GError* err = null;

		auto __p = as_pool_add_components(asPool, (cbox is null) ? null : cbox.getComponentBoxStruct(), &err) != 0;

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		return __p;
	}

	/**
	 * Add an additional non-standard location to the metadata pool where metadata will be read from.
	 * If @directory contains a "xml", "xmls", "yaml" or "icons" subdirectory (or all of them),
	 * those paths will be added to the search paths instead.
	 *
	 * Params:
	 *     directory = An existing filesystem location.
	 *     formatStyle = The expected format style of the metadata, e.g. %AS_FORMAT_STYLE_CATALOG
	 */
	public void addExtraDataLocation(string directory, AsFormatStyle formatStyle)
	{
		as_pool_add_extra_data_location(asPool, Str.toStringz(directory), formatStyle);
	}

	/**
	 * Convenience function to add one or multiple #AsPoolFlags to
	 * the flag set of this data pool.
	 *
	 * Params:
	 *     flags = The #AsPoolFlags to add.
	 *
	 * Since: 0.15.0
	 */
	public void addFlags(AsPoolFlags flags)
	{
		as_pool_add_flags(asPool, flags);
	}

	/**
	 * Splits up a string into an array of tokens that are suitable for searching.
	 * This includes stripping whitespaces, casefolding the terms and removing greylist words.
	 *
	 * This function is usually called automatically when needed, you will only need to
	 * run it explicitly when you need to check which search tokens the pool will actually
	 * use internally for a given phrase.
	 *
	 * Params:
	 *     search = the (user-provided) search phrase.
	 *
	 * Returns: (array zero-terminated=1): Valid tokens to search for, or %NULL for error
	 */
	public string[] buildSearchTokens(string search)
	{
		auto retStr = as_pool_build_search_tokens(asPool, Str.toStringz(search));

		scope(exit) Str.freeStringArray(retStr);
		return Str.toStringArray(retStr);
	}

	/**
	 * Remove all metadata from the pool, data will be reloaded
	 * once %as_pool_load is called again.
	 */
	public void clear()
	{
		as_pool_clear(asPool);
	}

	/**
	 * Get a list of found components.
	 *
	 * Returns: an #AsComponentBox.
	 */
	public ComponentBox getComponents()
	{
		auto __p = as_pool_get_components(asPool);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(ComponentBox)(cast(AsComponentBox*) __p, true);
	}

	/**
	 * Find components that are provided by a bundle with a specific ID by its prefix.
	 * For example, given a AS_BUNDLE_KIND_FLATPAK and a bundle_id "org.kde.dolphin/",
	 * it will list all the components that bundle dolphin. If the bundle_id is
	 * "org.kde.dolphin/x86_64" it will give those with also the architecture.
	 *
	 * Params:
	 *     kind = The kind of the bundle we are looking for
	 *     bundleId = The bundle ID to match, as specified in #AsBundle
	 *     matchPrefix = %TRUE to match the ID by prefix, %FALSE to perform an absolute match.
	 *
	 * Returns: an #AsComponentBox.
	 *
	 * Since: 0.16.0
	 */
	public ComponentBox getComponentsByBundleId(AsBundleKind kind, string bundleId, bool matchPrefix)
	{
		auto __p = as_pool_get_components_by_bundle_id(asPool, kind, Str.toStringz(bundleId), matchPrefix);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(ComponentBox)(cast(AsComponentBox*) __p, true);
	}

	/**
	 * Return a list of components which are in all of the categories.
	 *
	 * Params:
	 *     categories = An array of XDG categories to include.
	 *
	 * Returns: an #AsComponentBox of found components.
	 */
	public ComponentBox getComponentsByCategories(string[] categories)
	{
		auto __p = as_pool_get_components_by_categories(asPool, Str.toStringzArray(categories));

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(ComponentBox)(cast(AsComponentBox*) __p, true);
	}

	/**
	 * Find components extending the component with the given ID. They can then be registered to the
	 * #AsComponent they extend via %as_component_add_addon.
	 * If the %AS_POOL_FLAG_RESOLVE_ADDONS pool flag is set, addons are automatically resolved and
	 * this explicit function is not needed, but overall query time will be increased (so only use
	 * this flag if you will be resolving addon information later anyway).
	 *
	 * Params:
	 *     extendedId = The ID of the component to search extensions for.
	 *
	 * Returns: an #AsComponentBox.
	 *
	 * Since: 0.15.0
	 */
	public ComponentBox getComponentsByExtends(string extendedId)
	{
		auto __p = as_pool_get_components_by_extends(asPool, Str.toStringz(extendedId));

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(ComponentBox)(cast(AsComponentBox*) __p, true);
	}

	/**
	 * Get a specific component by its ID.
	 * This function may contain multiple results if we have
	 * data describing this component from multiple scopes/origin types.
	 *
	 * Params:
	 *     cid = The AppStream-ID to look for.
	 *
	 * Returns: an #AsComponentBox.
	 */
	public ComponentBox getComponentsById(string cid)
	{
		auto __p = as_pool_get_components_by_id(asPool, Str.toStringz(cid));

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(ComponentBox)(cast(AsComponentBox*) __p, true);
	}

	/**
	 * Return a list of all components in the pool which are of a certain kind.
	 *
	 * Params:
	 *     kind = An #AsComponentKind.
	 *
	 * Returns: an #AsComponentBox of found components.
	 */
	public ComponentBox getComponentsByKind(AsComponentKind kind)
	{
		auto __p = as_pool_get_components_by_kind(asPool, kind);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(ComponentBox)(cast(AsComponentBox*) __p, true);
	}

	/**
	 * Find components in the AppStream data pool which provide a specific launchable.
	 * See #AsLaunchable for details on launchables, or refer to the AppStream specification.
	 *
	 * Params:
	 *     kind = An #AsLaunchableKind
	 *     id = The ID of the launchable.
	 *
	 * Returns: an #AsComponentBox of found components.
	 *
	 * Since: 0.11.4
	 */
	public ComponentBox getComponentsByLaunchable(AsLaunchableKind kind, string id)
	{
		auto __p = as_pool_get_components_by_launchable(asPool, kind, Str.toStringz(id));

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(ComponentBox)(cast(AsComponentBox*) __p, true);
	}

	/**
	 * Find components in the AppStream data pool which provide a certain item.
	 *
	 * Params:
	 *     kind = An #AsProvidesKind
	 *     item = The value of the provided item.
	 *
	 * Returns: an #AsComponentBox of found components.
	 */
	public ComponentBox getComponentsByProvidedItem(AsProvidedKind kind, string item)
	{
		auto __p = as_pool_get_components_by_provided_item(asPool, kind, Str.toStringz(item));

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(ComponentBox)(cast(AsComponentBox*) __p, true);
	}

	/**
	 * Get the #AsPoolFlags for this data pool.
	 */
	public AsPoolFlags getFlags()
	{
		return as_pool_get_flags(asPool);
	}

	/**
	 * Gets the currently used locale.
	 *
	 * Returns: Locale used for metadata parsing.
	 */
	public string getLocale()
	{
		return Str.toString(as_pool_get_locale(asPool));
	}

	/**
	 * Check if this pool contains any data.
	 *
	 * Returns: %TRUE if the pool is empty.
	 */
	public bool isEmpty()
	{
		return as_pool_is_empty(asPool) != 0;
	}

	/**
	 * Builds an index of all found components in the watched locations.
	 * The function will try to get as much data into the pool as possible, so even if
	 * the update completes with %FALSE, it might still have added components to the pool.
	 *
	 * The function will load from all possible data sources, preferring caches if they
	 * are up to date.
	 *
	 * Params:
	 *     cancellable = a #GCancellable.
	 *
	 * Returns: %TRUE if update completed without error.
	 *
	 * Throws: GException on failure.
	 */
	public bool load(Cancellable cancellable)
	{
		GError* err = null;

		auto __p = as_pool_load(asPool, (cancellable is null) ? null : cancellable.getCancellableStruct(), &err) != 0;

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		return __p;
	}

	/**
	 * Asynchronously loads data from all registered locations.
	 * Equivalent to as_pool_load() (but asynchronous)
	 *
	 * Params:
	 *     cancellable = a #GCancellable.
	 *     callback = A #GAsyncReadyCallback
	 *     userData = Data to pass to @callback
	 *
	 * Since: 0.12.10
	 */
	public void loadAsync(Cancellable cancellable, GAsyncReadyCallback callback, void* userData)
	{
		as_pool_load_async(asPool, (cancellable is null) ? null : cancellable.getCancellableStruct(), callback, userData);
	}

	/**
	 * Retrieve the result of as_pool_load_async().
	 *
	 * Params:
	 *     result = A #GAsyncResult
	 *
	 * Returns: %TRUE for success
	 *
	 * Since: 0.12.10
	 *
	 * Throws: GException on failure.
	 */
	public bool loadFinish(AsyncResultIF result)
	{
		GError* err = null;

		auto __p = as_pool_load_finish(asPool, (result is null) ? null : result.getAsyncResultStruct(), &err) != 0;

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		return __p;
	}

	/**
	 * Convenience function to remove one or multiple #AsPoolFlags from
	 * the flag set of this data pool.
	 *
	 * Params:
	 *     flags = The #AsPoolFlags to remove.
	 *
	 * Since: 0.15.0
	 */
	public void removeFlags(AsPoolFlags flags)
	{
		as_pool_remove_flags(asPool, flags);
	}

	/**
	 * Remove all explicitly added metadata locations.
	 *
	 * Since: 0.15.0
	 */
	public void resetExtraDataLocations()
	{
		as_pool_reset_extra_data_locations(asPool);
	}

	/**
	 * Search for a list of components matching the search term.
	 * The list will be ordered by match score.
	 *
	 * Params:
	 *     search = A search string
	 *
	 * Returns: an #AsComponentBox of the found components.
	 *
	 * Since: 0.9.7
	 */
	public ComponentBox search(string search)
	{
		auto __p = as_pool_search(asPool, Str.toStringz(search));

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(ComponentBox)(cast(AsComponentBox*) __p, true);
	}

	/**
	 * Set the #AsPoolFlags for this data pool.
	 *
	 * Params:
	 *     flags = The new #AsPoolFlags.
	 */
	public void setFlags(AsPoolFlags flags)
	{
		as_pool_set_flags(asPool, flags);
	}

	/**
	 * This is a convenience function that enables or disables loading of metadata
	 * from well-known standard locations by configuring the #AsPoolFlags of this
	 * #AsPool accordingly.
	 * Data affected by this includes the OS data catalog, metainfo, desktop-entry
	 * files and Flatpak data.
	 * If you need more fine-grained control, set the #AsPoolFlags explicitly.
	 *
	 * Params:
	 *     enabled = Whether loading of data from standard locations should be enabled.
	 *
	 * Since: 0.15.0
	 */
	public void setLoadStdDataLocations(bool enabled)
	{
		as_pool_set_load_std_data_locations(asPool, enabled);
	}

	/**
	 * Sets the current locale which should be used when parsing metadata.
	 *
	 * Params:
	 *     locale = the BCP47 or POSIX locale to use for this pool.
	 */
	public void setLocale(string locale)
	{
		as_pool_set_locale(asPool, Str.toStringz(locale));
	}

	/**
	 * The ::changed signal is emitted when components have been added
	 * or removed from the metadata pool.
	 *
	 * Since: 0.15.0
	 */
	gulong addOnChanged(void delegate(Pool) dlg, ConnectFlags connectFlags=cast(ConnectFlags)0)
	{
		return Signals.connect(this, "changed", dlg, connectFlags ^ ConnectFlags.SWAPPED);
	}
}
