/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module appstream.ReleaseList;

private import appstream.Context;
private import appstream.Release;
private import appstream.c.functions;
public  import appstream.c.types;
private import glib.Bytes;
private import glib.ConstructionException;
private import glib.ErrorG;
private import glib.GException;
private import glib.PtrArray;
private import glib.Str;
private import glib.c.functions;
private import gobject.ObjectG;


/** */
public class ReleaseList : ObjectG
{
	/** the main Gtk struct */
	protected AsReleaseList* asReleaseList;

	/** Get the main Gtk struct */
	public AsReleaseList* getReleaseListStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return asReleaseList;
	}

	/** the main Gtk struct as a void* */
	protected override void* getStruct()
	{
		return cast(void*)asReleaseList;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (AsReleaseList* asReleaseList, bool ownedRef = false)
	{
		this.asReleaseList = asReleaseList;
		super(cast(GObject*)asReleaseList, ownedRef);
	}


	/** */
	public static GType getType()
	{
		return as_release_list_get_type();
	}

	/**
	 * Creates a new #AsReleaseList.
	 *
	 * Returns: an #AsReleaseList
	 *
	 * Since: 1.0
	 *
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this()
	{
		auto __p = as_release_list_new();

		if(__p is null)
		{
			throw new ConstructionException("null returned by new");
		}

		this(cast(AsReleaseList*) __p, true);
	}

	/**
	 * Append a release entry to this #AsReleaseList container.
	 */
	public void add(Release release)
	{
		as_release_list_add(asReleaseList, (release is null) ? null : release.getReleaseStruct());
	}

	/**
	 * Remove all release entries from this releases object.
	 */
	public void clear()
	{
		as_release_list_clear(asReleaseList);
	}

	/**
	 * Get the #AsContext associated with these releases.
	 * This function may return %NULL if no context is set
	 *
	 * Returns: the associated #AsContext or %NULL
	 */
	public Context getContext()
	{
		auto __p = as_release_list_get_context(asReleaseList);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(Context)(cast(AsContext*) __p);
	}

	/**
	 * Get the release entries as #GPtrArray.
	 *
	 * Returns: an array of #AsRelease instances.
	 */
	public PtrArray getEntries()
	{
		auto __p = as_release_list_get_entries(asReleaseList);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Returns the #AsReleaseListKind of the release metadata
	 * associated with this component.
	 *
	 * Returns: The kind.
	 *
	 * Since: 0.16.0
	 */
	public AsReleaseListKind getKind()
	{
		return as_release_list_get_kind(asReleaseList);
	}

	/**
	 * Get the amount of components in this box.
	 *
	 * Returns: Amount of components.
	 */
	public uint getSize()
	{
		return as_release_list_get_size(asReleaseList);
	}

	/**
	 * Get the remote URL to obtain release information from.
	 *
	 * Returns: The URL of external release data, or %NULL
	 *
	 * Since: 0.16.0
	 */
	public string getUrl()
	{
		return Str.toString(as_release_list_get_url(asReleaseList));
	}

	/**
	 * Retrieve a release entry at the respective index from the
	 * release entry list.
	 *
	 * Params:
	 *     index = The release entry index.
	 *
	 * Returns: An #AsRelease or %NULL
	 */
	public Release indexSafe(uint index)
	{
		auto __p = as_release_list_index_safe(asReleaseList, index);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(Release)(cast(AsRelease*) __p);
	}

	/**
	 * Check if there are any components present.
	 *
	 * Returns: %TRUE if this component box is empty.
	 */
	public bool isEmpty()
	{
		return as_release_list_is_empty(asReleaseList) != 0;
	}

	/**
	 * Load release information from XML bytes.
	 *
	 * Params:
	 *     context = the attached #AsContext or %NULL to use the current context
	 *     bytes = the release XML data as #GBytes
	 *
	 * Returns: %TRUE on success.
	 *
	 * Since: 0.16.0
	 *
	 * Throws: GException on failure.
	 */
	public bool loadFromBytes(Context context, Bytes bytes)
	{
		GError* err = null;

		auto __p = as_release_list_load_from_bytes(asReleaseList, (context is null) ? null : context.getContextStruct(), (bytes is null) ? null : bytes.getBytesStruct(), &err) != 0;

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		return __p;
	}

	/**
	 * Sets the document context these releases are associated with.
	 *
	 * Params:
	 *     context = the #AsContext.
	 */
	public void setContext(Context context)
	{
		as_release_list_set_context(asReleaseList, (context is null) ? null : context.getContextStruct());
	}

	/**
	 * Sets the #AsReleaseListKind of the release metadata
	 * associated with this component.
	 *
	 * Params:
	 *     kind = the #AsComponentKind.
	 *
	 * Since: 0.16.0
	 */
	public void setKind(AsReleaseListKind kind)
	{
		as_release_list_set_kind(asReleaseList, kind);
	}

	/**
	 * Set the amount of release entries stored.
	 */
	public void setSize(uint size)
	{
		as_release_list_set_size(asReleaseList, size);
	}

	/**
	 * Set a remote URL pointing to an AppStream release info file.
	 *
	 * Params:
	 *     url = the web URL where release data is found.
	 *
	 * Since: 0.16.0
	 */
	public void setUrl(string url)
	{
		as_release_list_set_url(asReleaseList, Str.toStringz(url));
	}

	/**
	 * Sort releases by their release version,
	 * starting with the most recent release.
	 */
	public void sort()
	{
		as_release_list_sort(asReleaseList);
	}
}
