/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module appstream.Screenshot;

private import appstream.Context;
private import appstream.Image;
private import appstream.Video;
private import appstream.c.functions;
public  import appstream.c.types;
private import glib.ConstructionException;
private import glib.PtrArray;
private import glib.Str;
private import glib.c.functions;
private import gobject.ObjectG;


/** */
public class Screenshot : ObjectG
{
	/** the main Gtk struct */
	protected AsScreenshot* asScreenshot;

	/** Get the main Gtk struct */
	public AsScreenshot* getScreenshotStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return asScreenshot;
	}

	/** the main Gtk struct as a void* */
	protected override void* getStruct()
	{
		return cast(void*)asScreenshot;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (AsScreenshot* asScreenshot, bool ownedRef = false)
	{
		this.asScreenshot = asScreenshot;
		super(cast(GObject*)asScreenshot, ownedRef);
	}


	/** */
	public static GType getType()
	{
		return as_screenshot_get_type();
	}

	/**
	 * Creates a new #AsScreenshot.
	 *
	 * Returns: a #AsScreenshot
	 *
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this()
	{
		auto __p = as_screenshot_new();

		if(__p is null)
		{
			throw new ConstructionException("null returned by new");
		}

		this(cast(AsScreenshot*) __p, true);
	}

	/**
	 * Adds an image to the screenshot.
	 *
	 * Params:
	 *     image = a #AsImage instance.
	 */
	public void addImage(Image image)
	{
		as_screenshot_add_image(asScreenshot, (image is null) ? null : image.getImageStruct());
	}

	/**
	 * Adds a video to the screenshot.
	 *
	 * Params:
	 *     video = a #AsVideo instance.
	 */
	public void addVideo(Video video)
	{
		as_screenshot_add_video(asScreenshot, (video is null) ? null : video.getVideoStruct());
	}

	/**
	 * Remove all images associated with this screenshot.
	 *
	 * Since: 0.15.4
	 */
	public void clearImages()
	{
		as_screenshot_clear_images(asScreenshot);
	}

	/**
	 * Gets the image caption
	 *
	 * Returns: the caption
	 */
	public string getCaption()
	{
		return Str.toString(as_screenshot_get_caption(asScreenshot));
	}

	/**
	 * Returns the #AsContext associated with this screenshot.
	 * This function may return %NULL if no context is set.
	 *
	 * Returns: the #AsContext used by this screenshot.
	 *
	 * Since: 0.11.2
	 */
	public Context getContext()
	{
		auto __p = as_screenshot_get_context(asScreenshot);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(Context)(cast(AsContext*) __p);
	}

	/**
	 * Get the GUI environment ID of this screenshot, if any
	 * is associated with it. E.g. "plasma-mobile" or "gnome:dark".
	 *
	 * Returns: The GUI environment ID the screenshot was recorded in, or %NULL if none set.
	 */
	public string getEnvironment()
	{
		return Str.toString(as_screenshot_get_environment(asScreenshot));
	}

	/**
	 * Gets the AsImage closest to the target size. The #AsImage may not actually
	 * be the requested size, and the application may have to pad / rescale the
	 * image to make it fit.
	 * Only images for the current active locale (or fallback, if images are not localized)
	 * are considered.
	 *
	 * Params:
	 *     width = target width
	 *     height = target height
	 *     scale = the target scaling factor.
	 *
	 * Returns: an #AsImage, or %NULL
	 *
	 * Since: 0.14.0
	 */
	public Image getImage(uint width, uint height, uint scale)
	{
		auto __p = as_screenshot_get_image(asScreenshot, width, height, scale);

		if(__p is null)
		{
			return null;
		}

		return ObjectG.getDObject!(Image)(cast(AsImage*) __p);
	}

	/**
	 * Gets the images for this screenshots. Only images valid for the current
	 * language are returned. We return all sizes.
	 *
	 * Returns: an array
	 */
	public PtrArray getImages()
	{
		auto __p = as_screenshot_get_images(asScreenshot);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Returns an array of all images we have, regardless of their
	 * size and language.
	 *
	 * Returns: an array
	 *
	 * Since: 0.10
	 */
	public PtrArray getImagesAll()
	{
		auto __p = as_screenshot_get_images_all(asScreenshot);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Gets the screenshot kind.
	 *
	 * Returns: a #AsScreenshotKind
	 */
	public AsScreenshotKind getKind()
	{
		return as_screenshot_get_kind(asScreenshot);
	}

	/**
	 * Gets the screenshot media kind.
	 *
	 * Returns: a #AsScreenshotMediaKind
	 */
	public AsScreenshotMediaKind getMediaKind()
	{
		return as_screenshot_get_media_kind(asScreenshot);
	}

	/**
	 * Gets the videos for this screenshots. Only videos valid for the current
	 * language selection are returned. We return all sizes.
	 *
	 * Returns: an array
	 */
	public PtrArray getVideos()
	{
		auto __p = as_screenshot_get_videos(asScreenshot);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Returns an array of all screencast videos we have, regardless of their
	 * size and locale.
	 *
	 * Returns: an array
	 */
	public PtrArray getVideosAll()
	{
		auto __p = as_screenshot_get_videos_all(asScreenshot);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Performs a quick validation on this screenshot
	 *
	 * Returns: TRUE if the screenshot is a complete #AsScreenshot
	 */
	public bool isValid()
	{
		return as_screenshot_is_valid(asScreenshot) != 0;
	}

	/**
	 * Sets a caption on the screenshot
	 *
	 * Params:
	 *     caption = the caption text.
	 */
	public void setCaption(string caption, string locale)
	{
		as_screenshot_set_caption(asScreenshot, Str.toStringz(caption), Str.toStringz(locale));
	}

	/**
	 * Sets the document context this screenshot is associated
	 * with.
	 *
	 * Params:
	 *     context = the #AsContext.
	 *
	 * Since: 0.11.2
	 */
	public void setContext(Context context)
	{
		as_screenshot_set_context(asScreenshot, (context is null) ? null : context.getContextStruct());
	}

	/**
	 * Sets the GUI environment ID of this screenshot.
	 *
	 * Params:
	 *     envId = the GUI environment ID, e.g. "plasma-mobile" or "gnome:dark"
	 */
	public void setEnvironment(string envId)
	{
		as_screenshot_set_environment(asScreenshot, Str.toStringz(envId));
	}

	/**
	 * Sets the screenshot kind.
	 *
	 * Params:
	 *     kind = the #AsScreenshotKind.
	 */
	public void setKind(AsScreenshotKind kind)
	{
		as_screenshot_set_kind(asScreenshot, kind);
	}
}
