/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module appstream.SystemInfo;

private import appstream.c.functions;
public  import appstream.c.types;
private import glib.ConstructionException;
private import glib.ErrorG;
private import glib.GException;
private import glib.PtrArray;
private import glib.Str;
private import glib.c.functions;
private import gobject.ObjectG;


/** */
public class SystemInfo : ObjectG
{
	/** the main Gtk struct */
	protected AsSystemInfo* asSystemInfo;

	/** Get the main Gtk struct */
	public AsSystemInfo* getSystemInfoStruct(bool transferOwnership = false)
	{
		if (transferOwnership)
			ownedRef = false;
		return asSystemInfo;
	}

	/** the main Gtk struct as a void* */
	protected override void* getStruct()
	{
		return cast(void*)asSystemInfo;
	}

	/**
	 * Sets our main struct and passes it to the parent class.
	 */
	public this (AsSystemInfo* asSystemInfo, bool ownedRef = false)
	{
		this.asSystemInfo = asSystemInfo;
		super(cast(GObject*)asSystemInfo, ownedRef);
	}


	/** */
	public static GType getType()
	{
		return as_system_info_get_type();
	}

	/**
	 * Creates a new #AsSystemInfo.
	 *
	 * Returns: a #AsSystemInfo
	 *
	 * Since: 0.10
	 *
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this()
	{
		auto __p = as_system_info_new();

		if(__p is null)
		{
			throw new ConstructionException("null returned by new");
		}

		this(cast(AsSystemInfo*) __p, true);
	}

	/**
	 * Creates a new #AsSystemInfo that can be used as a mock system
	 * for the given chassis type, reading (mostly) no information
	 * from the current system.
	 *
	 * This function will try to generate a device most representative of
	 * the given chassis class. This can be useful when testing compatibility
	 * for other devices on a different device that is not of the same chassis
	 * type as the one to be tested.
	 *
	 * Params:
	 *     chassis = the #AsChassisKind to generate a template for
	 *
	 * Returns: an #AsSystemInfo template
	 *
	 * Since: 1.0.0
	 *
	 * Throws: GException on failure.
	 * Throws: ConstructionException GTK+ fails to create the object.
	 */
	public this(AsChassisKind chassis)
	{
		GError* err = null;

		auto __p = as_system_info_new_template_for_chassis(chassis, &err);

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		if(__p is null)
		{
			throw new ConstructionException("null returned by new_template_for_chassis");
		}

		this(cast(AsSystemInfo*) __p, true);
	}

	/**
	 * Return a human readable device name for the given modalias.
	 * Will return the modalias again if no device name could be found,
	 * and returns %NULL on error.
	 * If @allow_fallback is set to %FALSE, this function will return %NULL and error
	 * %AS_SYSTEM_INFO_ERROR_NOT_FOUND in case no suitable description could be found.
	 *
	 * Params:
	 *     modalias = the modalias value to resolve (may contain wildcards).
	 *     allowFallback = fall back to low-quality data if no better information is available
	 *
	 * Returns: a human-readable device name, or %NULL on error.
	 *
	 * Throws: GException on failure.
	 */
	public string getDeviceNameForModalias(string modalias, bool allowFallback)
	{
		GError* err = null;

		auto retStr = as_system_info_get_device_name_for_modalias(asSystemInfo, Str.toStringz(modalias), allowFallback, &err);

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		scope(exit) Str.freeString(retStr);
		return Str.toString(retStr);
	}

	/**
	 * Get the current display length for the given side kind.
	 * If the display size is unknown, this function will return 0.
	 *
	 * Params:
	 *     side = the #AsDisplaySideKind to select.
	 *
	 * Returns: the display size in logical pixels.
	 */
	public gulong getDisplayLength(AsDisplaySideKind side)
	{
		return as_system_info_get_display_length(asSystemInfo, side);
	}

	/**
	 * Check whether graphical applications can be displayed via X11 or Wayland.
	 *
	 * Returns: %TRUE if graphical applications can be displayed.
	 */
	public bool getGuiAvailable()
	{
		return as_system_info_get_gui_available(asSystemInfo) != 0;
	}

	/**
	 * Get the name of the current kernel, e.g. "Linux"
	 *
	 * Returns: the current OS kernel name
	 */
	public string getKernelName()
	{
		return Str.toString(as_system_info_get_kernel_name(asSystemInfo));
	}

	/**
	 * Get the version of the current kernel, e.g. "6.2.0-2"
	 *
	 * Returns: the current kernel version
	 */
	public string getKernelVersion()
	{
		return Str.toString(as_system_info_get_kernel_version(asSystemInfo));
	}

	/**
	 * Get the current total amount of physical memory in MiB.
	 *
	 * Returns: the current total amount of usable memory in MiB
	 */
	public gulong getMemoryTotal()
	{
		return as_system_info_get_memory_total(asSystemInfo);
	}

	/**
	 * Get a list of modaliases for all the hardware on this system that has them.
	 *
	 * Returns: a list of modaliases on the system.
	 */
	public PtrArray getModaliases()
	{
		auto __p = as_system_info_get_modaliases(asSystemInfo);

		if(__p is null)
		{
			return null;
		}

		return new PtrArray(cast(GPtrArray*) __p);
	}

	/**
	 * Get the AppStream component ID of the current operating system.
	 *
	 * Returns: the component ID of the current OS.
	 */
	public string getOsCid()
	{
		return Str.toString(as_system_info_get_os_cid(asSystemInfo));
	}

	/**
	 * Get the homepage URL of the current operating system.
	 *
	 * Returns: the homepage of the current OS.
	 */
	public string getOsHomepage()
	{
		return Str.toString(as_system_info_get_os_homepage(asSystemInfo));
	}

	/**
	 * Get the ID of the current operating system.
	 *
	 * Returns: the current OS ID.
	 */
	public string getOsId()
	{
		return Str.toString(as_system_info_get_os_id(asSystemInfo));
	}

	/**
	 * Get the humen-readable name of the current operating system.
	 *
	 * Returns: the name of the current OS.
	 */
	public string getOsName()
	{
		return Str.toString(as_system_info_get_os_name(asSystemInfo));
	}

	/**
	 * Get the version string of the current operating system.
	 *
	 * Returns: the version of the current OS.
	 */
	public string getOsVersion()
	{
		return Str.toString(as_system_info_get_os_version(asSystemInfo));
	}

	/**
	 * Check if there is a device on this system that matches the given modalias glob.
	 *
	 * Params:
	 *     modaliasGlob = the modalias value to to look for, may contain wildcards.
	 *
	 * Returns: %TRUE if a matching device was found.
	 */
	public bool hasDeviceMatchingModalias(string modaliasGlob)
	{
		return as_system_info_has_device_matching_modalias(asSystemInfo, Str.toStringz(modaliasGlob)) != 0;
	}

	/**
	 * Test if the current system has a specific user input control method.
	 * Returns %AS_CHECK_RESULT_UNKNOWN if we could not test for an input control method,
	 * %AS_CHECK_RESULT_ERROR on error and %AS_CHECK_RESULT_FALSE if the control was not found.
	 *
	 * Params:
	 *     kind = the #AsControlKind to test for.
	 *
	 * Returns: %AS_CHECK_RESULT_TRUE if control was found
	 *
	 * Throws: GException on failure.
	 */
	public AsCheckResult hasInputControl(AsControlKind kind)
	{
		GError* err = null;

		auto __p = as_system_info_has_input_control(asSystemInfo, kind, &err);

		if (err !is null)
		{
			throw new GException( new ErrorG(err) );
		}

		return __p;
	}

	/**
	 * Receive a path in /sys for the devices with the given modalias.
	 *
	 * Params:
	 *     modalias = the modalias value to resolve.
	 *
	 * Returns: the syspath, or %NULL if none was found.
	 */
	public string modaliasToSyspath(string modalias)
	{
		return Str.toString(as_system_info_modalias_to_syspath(asSystemInfo, Str.toStringz(modalias)));
	}

	/**
	 * Set the current display length for the given side kind.
	 * The size needs to be in device-independent pixels, see the
	 * AppStream documentation for more information:
	 * https://freedesktop.org/software/appstream/docs/chap-Metadata.html#tag-relations-display_length
	 *
	 * Params:
	 *     side = the #AsDisplaySideKind to select.
	 *     valueDip = the length value in device-independt pixels.
	 */
	public void setDisplayLength(AsDisplaySideKind side, gulong valueDip)
	{
		as_system_info_set_display_length(asSystemInfo, side, valueDip);
	}

	/**
	 * Set whether this system has a GUI / desktop environment available.
	 *
	 * Params:
	 *     available = %TRUE if GUI is available.
	 */
	public void setGuiAvailable(bool available)
	{
		as_system_info_set_gui_available(asSystemInfo, available);
	}

	/**
	 * Explicitly mark a user input control as present or not present on this system.
	 *
	 * Params:
	 *     kind = the #AsControlKind to set.
	 *     found = %TRUE if the control should be marked as found.
	 */
	public void setInputControl(AsControlKind kind, bool found)
	{
		as_system_info_set_input_control(asSystemInfo, kind, found);
	}
}
