/*
 * Licensed under the GNU Lesser General Public License Version 3
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

// generated automatically - do not change


module appstream.Utils;

private import appstream.c.functions;
public  import appstream.c.types;
private import glib.Str;
private import glib.c.functions;


/** */
public struct Utils
{

	/**
	 * Get the version of the AppStream library that is currently used
	 * at runtime as a string.
	 *
	 * Returns: a version number, e.g. "0.14.2"
	 *
	 * Since: 0.14.0
	 */
	public static string appstreamVersion()
	{
		return Str.toString(as_version_string());
	}

	/**
	 * Converts the enumerated value to an text representation.
	 *
	 * Params:
	 *     severity = the #AsIssueSeverity.
	 *
	 * Returns: string version of @severity
	 */
	public static string severityToString(AsIssueSeverity severity)
	{
		return Str.toString(as_issue_severity_to_string(severity));
	}

	/**
	 * Converts the text representation to an enumerated value.
	 *
	 * Params:
	 *     str = the string.
	 *
	 * Returns: a #AsIssueSeverity, or %AS_ISSUE_SEVERITY_UNKNOWN for unknown.
	 */
	public static AsIssueSeverity severityFromString(string str)
	{
		return as_issue_severity_from_string(Str.toStringz(str));
	}

	/**
	 * Searches the known list of TLDs we allow for AppStream IDs.
	 * This excludes internationalized names.
	 *
	 * Params:
	 *     tld = a top-level domain without dot, e.g. "de", "org", "name"
	 *
	 * Returns: %TRUE if the TLD is valid
	 *
	 * Since: 0.9.8
	 */
	public static bool isTld(string tld)
	{
		return as_utils_is_tld(Str.toStringz(tld)) != 0;
	}

	/**
	 * Searches the known list of registered XDG category names.
	 * See https://specifications.freedesktop.org/menu-spec/menu-spec-1.0.html#category-registry
	 * for a reference.
	 *
	 * Params:
	 *     categoryName = a XDG category name, e.g. "ProjectManagement"
	 *
	 * Returns: %TRUE if the category name is valid
	 *
	 * Since: 0.9.7
	 */
	public static bool isCategoryName(string categoryName)
	{
		return as_utils_is_category_name(Str.toStringz(categoryName)) != 0;
	}

	/**
	 * Converts the enumerated value to an text representation.
	 *
	 * Params:
	 *     version_ = the #AsFormatKind.
	 *
	 * Returns: string version of @version
	 *
	 * Since: 0.10.0
	 */
	public static string formatVersionToString(AsFormatVersion version_)
	{
		return Str.toString(as_format_version_to_string(version_));
	}

	/**
	 * Converts the text representation to an enumerated value.
	 *
	 * Params:
	 *     versionStr = the string.
	 *
	 * Returns: a #AsFormatVersion. For unknown, the highest version
	 *     number is assumed.
	 *
	 * Since: 0.10.0
	 */
	public static AsFormatVersion formatVersionFromString(string versionStr)
	{
		return as_format_version_from_string(Str.toStringz(versionStr));
	}

	/**
	 * Check if the metadata license is suitable for mixing with other
	 * metadata and redistributing the bundled result (this means we
	 * prefer permissive licenses here, to not require people shipping
	 * catalog metadata to perform a full license review).
	 *
	 * This method checks against a hardcoded list of permissive licenses
	 * commonly used to license metadata under.
	 *
	 * Params:
	 *     license = The SPDX license string to test.
	 *
	 * Returns: %TRUE if the license contains only permissive licenses suitable
	 *     as metadata license.
	 */
	public static bool licenseIsMetadataLicense(string license)
	{
		return as_license_is_metadata_license(Str.toStringz(license)) != 0;
	}

	/**
	 * Tokenizes the SPDX license string (or any simarly formatted string)
	 * into parts. Any license parts of the string e.g. "LGPL-2.0+" are prefexed
	 * with "@", the conjunctive replaced with "&", the disjunctive replaced
	 * with "|" and the WITH operator for license exceptions replaced with "^".
	 * Brackets are added as indervidual tokens and other strings are
	 * appended into single tokens where possible.
	 *
	 * Params:
	 *     license = a license string, e.g. "LGPLv2+ and (QPL or GPLv2) and MIT"
	 *
	 * Returns: array of strings, or %NULL for invalid
	 *
	 * Since: 0.9.8
	 */
	public static string[] spdxLicenseTokenize(string license)
	{
		auto retStr = as_spdx_license_tokenize(Str.toStringz(license));

		scope(exit) Str.freeStringArray(retStr);
		return Str.toStringArray(retStr);
	}

	/**
	 * Converts the enumerated value to an text representation.
	 *
	 * Params:
	 *     kind = the #AsComponentKind.
	 *
	 * Returns: string version of @kind
	 */
	public static string componentKindToString(AsComponentKind kind)
	{
		return Str.toString(as_component_kind_to_string(kind));
	}
}
